{*
 * Projecte Fressa a LINKAT
 * FUNCIONS WIN 32
 *
 * Author Jordi Lagares Roset "jlagares@xtec.cat - www.lagares.org"
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details (see the LICENSE file).
 *}

unit UnitCalcularFuncioNumerica;

{*******************************************************************************}
interface
{*******************************************************************************}

Procedure AnalitzarFuncioNumerica(var i:integer);
Procedure CalcularFuncionumerica(var xNum,yNum: real; var ErrorsN:integer);
Function EsFuncioNumerica(a:string):boolean;

{*******************************************************************************}
implementation
{*******************************************************************************}

uses UnitVariables;

type
   Coef = array[1..7,1..NumericaMax] of real;

var
    xt:Coef;
    UlLloc:integer;
    Quina:integer;

Procedure AnalitzarFuncioNumerica(var i:integer);
var
  l,n: integer;
  D1,D2,D3,D4: real;
  H1,H2,H3,H4,H5,H6,H7: real;
begin
  Quina:=i;
  UlLloc:=UltimLloc[i];
  l:=UlLloc;
  for n:=1 to l do begin
    xt[1,n]:= FN[i,1,n];
    xt[2,n]:= FN[i,2,n];
  end;
  if (EsLineal[i]) or (l=2) then begin
    for n:=1 to l-1 do begin
      xt[3,n]:=0;
      xt[4,n]:=0;
      xt[5,n]:=(FN[i,2,n+1]-FN[i,2,n])/(FN[i,1,n+1]-FN[i,1,n]);
      xt[6,n]:=FN[i,2,n]-FN[i,1,n]*xt[5,n];
    end;
  end else begin
    D1:=(xt[2,2]-xt[2,1])/(xt[1,2]-xt[1,1]);
    D2:=(xt[2,3]-xt[2,2])/(xt[1,3]-xt[1,2]);
    D3:=(xt[1,2]-xt[1,1])/(xt[1,3]-xt[1,1]);
    D4:=(xt[1,2]-xt[1,1])/2;
    xt[7,1]:=D1+(D1-D2)*D3;
    for n:=1 to l-2  do begin
      H1:=xt[2,n+1];
      H2:=xt[2,n];
      H3:=xt[7,n];
      H4:=xt[1,n+1];
      H5:=xt[1,n];
      H6:=xt[1,n+2];
      H7:=xt[2,n+2];
      xt[3,n]:=(H1-H2-H3*(H4-H5))*((H6-H5)*(H6-H5))-((H4-H5)*(H4-H5))
       *(H7-H2-H3*(H6-H5));
      xt[3,n]:=xt[3,n]/((H4*H4*H4-3*(H5*H5)*H4+2*(H5*H5*H5))
       *((H6-H5)*(H6-H5))-((H4-H5)*(H4-H5))*(H6*H6*H6-3*(H5*H5)
       *H6+2*H5*H5*H5));
      xt[4,n]:=((H7-H2-H3*(H6-H5))-xt[3,n]*((H6 * H6 * H6)
       -3*(H5*H5)*H6+2*(H5*H5*H5)))/((H6-H5)*(H6-H5));
       xt[5,n]:=H3-3*(H5*H5)*xt[3,n]-2*H5*xt[4,n];
       xt[6,n]:=H2-H3*H5+2*(H5*H5*H5)*xt[3,n]+(H5*H5)*xt[4,n];
       xt[7,n+1]:=3*xt[3,n]*(H4*H4)+2*xt[4,n]*H4+xt[5,n];
    end;
  end;
end;

Procedure CalcularFuncionumerica(var xNum,yNum: real; var ErrorsN:integer);
var
  l,n,k:integer;
  x2,x3:real;
  a1,a2,a3,a4:real;
begin
  if EsLineal[Quina] then l:=UlLloc-1 else l:=UlLloc-2;
  if ((xNum<xt[1,1]) OR (xNum>xt[1,UlLloc])) and
   (EsExtrapolar[Quina]=false) then begin
    yNum:=0;
    ErrorsN:= 101; {fora limits}
  end else begin
    ErrorsN:=0;
    k:=1;
    for n:=1 to l do begin
      k:=n;
      //if xNum<xt[1,n+1] then n:=l;
      if xNum<xt[1,n+1] then break;
    end;
    if (k>l) and (l>0) then k:=l;
    x2:=xNum*xNum;x3:=x2*xNum;
    a1:=xt[3,k];a2:=xt[4,k];a3:=xt[5,k];a4:=xt[6,k];
    yNum:=a1*x3+a2*x2+a3*xNum+a4;
  end;
end;

{*******************************************************************************}
Function EsFuncioNumerica(a:string):boolean;
begin
  //V:15 02/01/06
  //if (a=FunNumerica) or (a=FunInterLin) or (a=FunReLineal) or (a=FunReCuad) or (a=FunReEx) or (a=FunReEx2) or (a=FunReInv) or (a=FunNumericaE) or (a=FunInterLinE) or (a=FunReLinealE) or (a=FunReCuadE) or (a=FunReExE) or (a=FunReEx2E) or (a=FunReInvE) then EsFuncioNumerica:=true else EsFuncioNumerica:=false;
  if (a=FunNumerica) or (a=FunInterLin) or (a=FunReLineal) or (a=FunReCuad) or (a=FunReEx) or (a=FunReEx2) or (a=FunReInv) then EsFuncioNumerica:=true else EsFuncioNumerica:=false;
end;

end.
